#!/bin/bash

###
# Set me
###
SAS2IRCU=/usr/local/sbin/sas2ircu

CCOUNT=0
aOPT=0
AOPT=0
lOPT=0
LOPT=0

SCRIPTNAME=$(basename $0)

unset DISK
unset SAS_DSPY

###
# Usage
###
usage()
{
  cat <<EOF
Description: Map OS disk names to LSI controller index
             number, enclosure and slot. Optionally
             turn on/off either all or selected disk
             LED(s).

Usage: ${SCRIPTNAME} [option]

Options:

 -A          : Turn all LEDs on
 -a          : Turn all LEDs off
 -h          : Show this help
 -L <disk>   : Disk to turn LED on
 -l <disk>   : Disk to turn LED off
EOF
}


###
# Initialize LSI HBA list:
#   count the number of HBAs
#   associate each HBAs PCI address with the corresponding index num
#   concatenate sas2ircu DISPLAY output for each HBA
###
init_list()
{
  SAS_LIST=$(${SAS2IRCU} LIST | grep "1000h")

  while read CNTR CTYPE CVEND CID ADDR CREM ; do

    CCOUNT=$(expr ${CCOUNT} + 1)

    SAS_DSPY="${SAS_DSPY}"$'\n'"$(sas_dspy ${CNTR})"

  done <<<"${SAS_LIST}"
}


###
# Return display controller output
###
sas_dspy()
{
  $SAS2IRCU $1 DISPLAY
}


###
# Extract disk WWN ID and corresponding controller PCI ID device path from udev DB
# xref the controller index number from sas2ircu LIST output using PCI ID
# Extract Enclosure and Slot numbers from sas2ircu * DISPLAY
###
get_disk_info()
{
  UDEV_PROP=$(udevadm info --query=property ${1} | egrep "ID_WWN=0x|ID_PATH=" | sort)

  FS=$(echo $UDEV_PROP | sed 's/[^\ ]//g'|wc -m)

  if [ ${FS} -eq 2 ]; then

    read ID_PATH ID_WWN <<< $UDEV_PROP
    
    ID=$(echo $ID_PATH | awk -F: '{ print $2 }')

    GUID=$(echo $ID_WWN | sed -e 's/^ID_WWN=0x//1')

    CNTR_IDX=$(echo "${SAS_LIST}" | awk -v ID=":${ID}h:" ' $0 ~ ID { print $1 }')

    DSPLY_FILT=$(echo "${SAS_DSPY}"| grep -B 9 ${GUID} | awk '/Enclosure|Slot/ { print $4 }')

    read ENC SLOT <<< ${DSPLY_FILT}

  else unset DISK

  fi
}


###
# main()
###

if [ $EUID -ne 0 ]; then
  usage
  printf "\nThis script should be run as root.\n" 1>&2
  exit 1
fi

if [ $# -gt 2 ]; then
  usage
  printf "\nToo many arguments.\n"
  exit 1
fi 

while getopts ":L:l:Aah?" OPTIONS; do
    case "${OPTIONS}" in
        L)
            LOPT=$(expr ${LOPT} + 1)
            DISK=${OPTARG}
            LED="ON"
            ;;
        l)
            lOPT=$(expr ${lOPT} + 1)
            DISK=${OPTARG}
            LED="OFF"
            ;;
        A)
            AOPT=$(expr ${AOPT} + 1)
            LED="ON"
            ;;
        a)
            aOPT=$(expr ${aOPT} + 1)
            LED="OFF"
            ;;
        h)  usage; exit 0
            ;;
        ?)  usage; exit 1
            ;;
        *)  usage; exit 1
            ;;
    esac
done

TOPT=$(expr ${LOPT} + ${lOPT} + ${AOPT} + ${aOPT})

if [ ${TOPT} -gt 1 ]; then
  usage; exit 1
fi

init_list

if [ -n "${DISK}" ]; then

  get_disk_info ${DISK}

  if [ -n "${DISK}" ]; then
    ${SAS2IRCU} ${CNTR_IDX} LOCATE ${ENC}:${SLOT} ${LED} >/dev/null && printf "${DISK}\tLED ${LED}\n"
  else
   echo "Not an LSI controlled disk"; exit 1
  fi

elif [ ${AOPT} -eq 1 ] || [ ${aOPT} -eq 1 ] ; then
  for DISK in `ls -1 /dev/sd*[a-z]`; do
    get_disk_info ${DISK}
    if [ -n "${DISK}" ]; then
      ${SAS2IRCU} ${CNTR_IDX} LOCATE ${ENC}:${SLOT} ${LED} >/dev/null && printf "${DISK}\tLED ${LED}\n"
    fi
  done

else
  for DISK in `ls -1 /dev/sd*[a-z]`; do
    get_disk_info ${DISK}
    if [ -n "${DISK}" ]; then
      printf "${DISK}\t${GUID}\t${CNTR_IDX}\t${ENC}\t${SLOT}\n"
    fi
  done

fi
